import pickle
import os

files_path = os.listdir('./stats')

for file_path in files_path:
    with open('./stats/' + file_path, 'rb') as f:
        try:
            while True:
                print(pickle.load(f))
        except EOFError:
            pass



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import arcade
from arcade.gui import UIManager
from arcade.gui.ui_style import UIStyle
import mido
from miditeach.views.GameView import GameView
from pathlib import Path

SCREEN_WIDTH = 1000
SCREEN_HEIGHT = 500

class Selector(arcade.gui.UIImageToggle):
    def __init__(self, center_x, center_y):
        self.base_path = str(Path(__file__).absolute().parent.parent)
        select_off = arcade.load_texture(self.base_path + "/assets/images/select_off.png")
        select_on = arcade.load_texture(self.base_path + "/assets/images/select_on.png")
        super().__init__(center_x=center_x, center_y=center_y, true_texture=select_on, false_texture=select_off)
    
    def on_click(self):
        self.value = not self.value


class WelcomeScreen(arcade.View):
    """ Welcome screen and credits """

    def __init__(self):
        super().__init__()
        self.input_select = 0
        self.ui_manager = UIManager()

    def on_show(self):
        """ This is run once when we switch to this view """
        arcade.set_background_color(arcade.csscolor.BLACK)

    def on_hide_view(self):
        self.ui_manager.unregister_handlers()

    def get_selector_status(self):
        d = {}
        for chord in self.buttons:
            d[chord] = self.buttons[chord].value
        return d

    def setup(self):
        self.buttons = {}
        for e, chord in enumerate(['min', 'maj', 'dim', 'aug', 'm7', 'M7', '7']):
            self.buttons[chord] = Selector(SCREEN_WIDTH* (1-1/6), SCREEN_HEIGHT / 4 + 36 * e)
            label = arcade.gui.UILabel(center_x = SCREEN_WIDTH* (1-1/6) + 50, center_y = SCREEN_HEIGHT / 4 + 36 * e, text = chord, align = "right")
            self.ui_manager.add_ui_element(label)
            self.ui_manager.add_ui_element(self.buttons[chord])

    def on_draw(self):
        """ Draw this view """
        arcade.start_render()
        arcade.draw_text("midiTeacher V0.5", SCREEN_WIDTH / 2, SCREEN_HEIGHT * (1-1/4),
                         arcade.color.WHITE, font_size=30, anchor_x="center")
        arcade.draw_text("Alexis LOUIS - github.com/alelouis", SCREEN_WIDTH-15, 15,
                         arcade.color.WHITE, font_size=15, anchor_x="right")
        arcade.draw_text("Select MIDI input (UP/DOWN)", SCREEN_WIDTH / 2, SCREEN_HEIGHT * (1-1/4) - 40,
                         arcade.color.WHITE, font_size=20, anchor_x="center")
        arcade.draw_text("(Press ENTER to start)", SCREEN_WIDTH / 2, SCREEN_HEIGHT * (1/4) - 40,
                    arcade.color.YELLOW, font_size=20, anchor_x="center")

        inputs = mido.get_input_names()
        for e, i in enumerate(inputs):
            color = arcade.color.GREEN if e == self.input_select else arcade.color.WHITE
            arcade.draw_text(f'({e+1}) '+ i, SCREEN_WIDTH / 2 - SCREEN_WIDTH/8, SCREEN_HEIGHT * (1-1/4) - (e+5)*20,
                         color, font_size=18, anchor_x="left")

        
    def on_show_view(self):
        """ Show this view """
        self.setup()

    def on_key_press(self, key, modifiers):
        """ If the user presses the mouse button, start the game. """
        if key == arcade.key.DOWN:
            self.input_select += 1
            self.input_select = min(self.input_select, len(mido.get_input_names()))
        if key == arcade.key.UP:
            self.input_select -= 1
            self.input_select = max(self.input_select, 0)
        if key == arcade.key.RETURN:
            game_view = GameView(input_select = self.input_select, chords_selected = self.get_selector_status())
            game_view.setup()
            self.window.show_view(game_view)


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import arcade
from miditeach.views.WelcomeScreen import WelcomeScreen

SCREEN_WIDTH = 1000
SCREEN_HEIGHT = 500

def main():
    window = arcade.Window(SCREEN_WIDTH, SCREEN_HEIGHT, "midiTeacher")
    start_view = WelcomeScreen()
    window.show_view(start_view)
    arcade.run()


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import arcade

SCREEN_WIDTH = 1000
SCREEN_HEIGHT = 500

class PauseView(arcade.View):
    """ Pause view, resumes, resets and quits game """
    def __init__(self, game_view):
        super().__init__()
        self.game_view = game_view

    def on_show(self):
        """ This is run once when we switch to this view """
        arcade.set_background_color(arcade.csscolor.BLACK)

    def on_draw(self):
        """ Draw this view """
        arcade.start_render()
        
        arcade.draw_text("Press Q to quit.\nPress R to restart.", 
            SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2,
            arcade.color.WHITE, font_size=20, anchor_x="center")

    def on_key_press(self, key, modifiers):
        """ Quit or resume game. """
        if key == arcade.key.Q:
            self.window.close()
            self.game_view.stats_file.close()
        if key == arcade.key.R:
            self.game_view.reset()
            self.window.show_view(self.game_view)



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import mido
import arcade
import random
import time
import pickle
from datetime import datetime
from miditeach.views.PauseView import PauseView
from pathlib import Path

SCREEN_WIDTH = 1000
SCREEN_HEIGHT = 500

class GameView(arcade.View):
    """ Main application class. """

    def __init__(self, input_select, chords_selected):
        super().__init__()
        self.played_notes = [0]*12
        self.notes = ['C', 'C#\nDb', 'D', 'D#\nEb', 'E', 'F', 'F#\nGb', 'G', 'G#\nAb', 'A', 'A#\nBb', 'B']
        self.intervals = ['1', '2m', '2', '3m', '3', '4', 'T', '5', '6m', '6', '7m', '7']
        self.all_formulas = {
            'min':['1', '3m', '5'],
            'maj':['1', '3', '5'],
            'dim':['1', '3m', 'T'],
            'aug':['1', '3', '6m'],
            'm7':['1', '3m', '5', '7m'],
            'M7':['1', '3', '5', '7'],
            '7':['1', '3', '5', '7m']}

        self.formulas = {k:self.all_formulas[k] for k in chords_selected}
        self.base_path = str(Path(__file__).absolute().parent.parent)
        self.selected_device = mido.get_input_names()[input_select]
        self.stats_path = self.base_path + '/stats/' + datetime.now().strftime('%m%d%Y') + '.p'
        self.stats_file = open(self.stats_path, 'wb')

        self.inport = mido.open_input(self.selected_device)
        self.sample_next_chord()
        self.reset()


    def reset(self):
        """ Reset time and accuracy metrics """
        self.total_duration = 0
        self.total_chords = 0
        self.total_correct = 0
        self.total_incorrect = 0
        self.t_launch = datetime.now()


    def check_midi(self):
        """ Checks incoming midi data """
        for msg in self.inport.iter_pending(): 
            if msg.type == 'note_on':
                self.played_notes[msg.note%12] += 1
            if msg.type == 'note_off':
                self.played_notes[msg.note%12] -= 1


    def sample_next_chord(self):
        """ Sample a new root and formula """
        # Formula
        self.formula_name = random.choice(list(self.formulas.keys()))
        self.formula = self.formulas[self.formula_name]
        
        # Root
        self.root = random.choice(self.notes)
        self.root_select = random.choice(self.root.split('\n'))
        self.root_text = self.root_select + self.formula_name

        # Expected notes
        self.expected_notes = [0]*12
        for interval in self.formula: 
            self.expected_notes[(self.notes.index(self.root) + self.intervals.index(interval))%12] = 1
        

    def get_stat_dict(self):
        st = {}
        st['root'] = self.root_select
        st['formula'] = self.formula_name
        st['time'] = self.last_duration
        st['timestamp'] = datetime.now()
        return st

    def check_next(self):
        """ Checks if user misses or validate a chord """
        if self.is_incorrect() or self.is_correct():
            self.total_chords += 1

            if self.is_correct():
                arcade.play_sound(self.correct_sound)
            else:
                arcade.play_sound(self.wrong_sound)

            # Accuracy metrics
            self.total_correct += self.is_correct()
            self.total_incorrect += self.is_incorrect()

            # Time metrics
            self.t_end = datetime.now()
            self.last_duration = self.t_end - self.t_start
            self.last_duration = float(str(self.last_duration).split(':')[-1][1:6])
            self.total_duration += self.last_duration
            
            # Saving stats
            pickle.dump(self.get_stat_dict(), self.stats_file)

            # Next chords
            time.sleep(0.5)
            self.t_start = datetime.now()
            self.sample_next_chord()


    def is_incorrect(self):
        """ Checks if user inputs a wrong note """
        correct_notes = sum([a and b for a,b in zip(self.expected_notes, self.played_notes)])
        incorrect_notes = sum([a>0 for a in self.played_notes]) - correct_notes
        is_incorrect = incorrect_notes > 0
        return is_incorrect


    def is_correct(self):
        """ Checks if user inputs a correct chord """
        correct_notes = sum([a and b for a,b in zip(self.expected_notes, self.played_notes)])
        is_correct = correct_notes == len(self.formula)
        return is_correct


    def setup(self):
        """ Setup called before app run """
        arcade.set_background_color(arcade.color.BLACK)
        self.t_start = datetime.now()
        self.last_duration = datetime.now()-datetime.now()
        self.correct_sound = arcade.load_sound(self.base_path + "/assets/sounds/correct.wav")
        self.wrong_sound = arcade.load_sound(self.base_path + "/assets/sounds/wrong.wav")


    def on_draw(self):
        """ Render the screen """
        arcade.start_render()

        # Keyboard
        for note_index in range(12):
            if self.played_notes[note_index]:
                if self.expected_notes[note_index]: color = arcade.color.GREEN
                else: color = arcade.color.RED
            else: color = arcade.color.GRAY

            arcade.draw_text(self.notes[note_index], 
                SCREEN_WIDTH/24 + note_index * SCREEN_WIDTH/12 , 
                SCREEN_HEIGHT-SCREEN_HEIGHT/4, 
                color, 30, 
                anchor_x='center', anchor_y='center', font_name = 'Consolas')

        # Intervals
        for i, interval in enumerate(self.formula):
            interval_index = (self.intervals.index(interval) + self.notes.index(self.root))%12
            interval_color = arcade.color.GREEN if self.played_notes[interval_index] else arcade.color.GRAY
            offset = 0 if len(self.formula)%2 else 0.5
            arcade.draw_text(interval,
                SCREEN_WIDTH/2 + (i+ offset -len(self.formula)//2)*(SCREEN_WIDTH/8), 
                SCREEN_HEIGHT/2-SCREEN_HEIGHT/4, 
                interval_color, 50, 
                anchor_x='center', anchor_y='center', font_name = 'Consolas')

        # Root
        root_color = arcade.color.GREEN if self.is_correct() else arcade.color.RED if self.is_incorrect() else arcade.color.GRAY
        arcade.draw_text(self.root_text, 
            SCREEN_WIDTH/2 , 
            SCREEN_HEIGHT/2, 
            root_color, 70, 
            anchor_x='center', anchor_y='center', font_name = 'Consolas') 

        # Duration
        last_duration_text = 'undefined' if self.total_chords==0 else '%.2f'%self.last_duration
        mean_duration_text = 'undefined' if self.total_chords==0 else '%.2f'%(self.total_duration/self.total_chords)
        arcade.draw_text('last : ' + last_duration_text + ' s', 
            SCREEN_WIDTH*(1-1/12),
            SCREEN_HEIGHT/2, 
            root_color, 20, 
            anchor_x='right', anchor_y='center', font_name = 'Consolas')
        arcade.draw_text('mean: ' + mean_duration_text + ' s', 
            SCREEN_WIDTH*(1-1/12), 
            SCREEN_HEIGHT/2 - 25, 
            root_color, 20, 
            anchor_x='right', anchor_y='center', font_name = 'Consolas')    

        # Correct and wrong
        arcade.draw_text('correct : ' + str(self.total_correct), 
            SCREEN_WIDTH*(1/12),
            SCREEN_HEIGHT/2, 
            root_color, 20, 
            anchor_x='left', anchor_y='center', font_name = 'Consolas')
        arcade.draw_text('wrong : ' + str(self.total_incorrect), 
            SCREEN_WIDTH*(1/12), 
            SCREEN_HEIGHT/2 - 25, 
            root_color, 20, 
            anchor_x='left', anchor_y='center', font_name = 'Consolas')    

        # Timer
        elapsed_mins = (datetime.now() - self.t_launch).seconds//60
        elapsed_secs = (datetime.now() - self.t_launch).seconds%60
        elapsed_microsecs = (datetime.now() - self.t_launch).microseconds/10000
        arcade.draw_text('%02d:%02d:%02d'%(elapsed_mins,elapsed_secs,elapsed_microsecs), 
            SCREEN_WIDTH/2, SCREEN_HEIGHT*(1-1/10), 
            root_color, 20, 
            anchor_x='center', anchor_y='center', font_name = 'Consolas')  

        # Shortcuts
        arcade.draw_text('(Press SPACE to stop)', 
            SCREEN_WIDTH/2, SCREEN_HEIGHT*(1/10), 
            root_color, 10, 
            anchor_x='center', anchor_y='center', font_name = 'Consolas')  


    def on_key_press(self, key, modifiers):
        """Called whenever a key is pressed. """

        # If the player presses a key, update the speed
        if key == arcade.key.SPACE:
            pause_view = PauseView(self)
            self.window.show_view(pause_view)


    def update(self, delta_time):
        """ Game update """
        self.check_next()
        self.check_midi()




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['miditeach', 'miditeach.views']

package_data = \
{'': ['*'], 'miditeach': ['assets/images/*', 'assets/sounds/*', 'stats/*']}

install_requires = \
['arcade>=2.5.7,<3.0.0', 'mido>=1.2.10,<2.0.0', 'python-rtmidi>=1.4.9,<2.0.0']

entry_points = \
{'console_scripts': ['miditeach = miditeach.miditeach:main']}

setup_kwargs = {
    'name': 'miditeach',
    'version': '0.6',
    'description': '',
    'long_description': None,
    'author': 'Alexis LOUIS',
    'author_email': 'alelouis.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
